from buzio import console, formatStr
from dashing import dashing
from tabulate import tabulate
from cabrita.core.parser import Config
from cabrita.core.utils import get_path
import os

DEFAULT_DICT = {
    'compose': {
        'old_services_count': None,
        'new_services_count': None
    }
}


def _check_git(path):
    if not os.path.isdir(os.path.join(path, ".git")):
        ret = "ok"
    else:
        ret = console.run(
            "cd {} && git fetch && git status -bs --porcelain".format(path),
            get_stdout=True
        )
    if not ret:
        text = formatStr.warning(
            "NOT FOUND",
            use_prefix=False)
    elif 'behind' in ret:
        text = formatStr.error(
            'NEED PULL',
            use_prefix=False)
    else:
        text = formatStr.success(
            'OK',
            use_prefix=False)
    return text


def _check_timestamp(datadict, base_path):
    dest_full_path = os.path.join(
        get_path(datadict['dest_path'], base_path), datadict['name'])
    source_name = datadict.get('source_name', datadict['name'])
    source_full_path = os.path.join(
        get_path(datadict['source_path'], base_path), source_name)

    if not os.path.isfile(
            dest_full_path) or not os.path.isfile(source_full_path):
        return formatStr.warning(
            'NOT FOUND',
            use_prefix=False
        )

    source_date = os.path.getmtime(source_full_path)
    dest_date = os.path.getmtime(dest_full_path)

    if source_date > dest_date:
        text = formatStr.error(
            'NEED UPDATE',
            use_prefix=False)
    else:
        text = formatStr.success(
            'OK',
            use_prefix=False)
    return text


def get_check_status(dash):

    # Check for status file
    config = Config()
    status_full_path = get_path(dash.status_file_path, dash.path)
    if not os.path.isdir(status_full_path):
        os.makedirs(status_full_path)
    status_path = os.path.join(status_full_path, "status.yml")
    if config.check_file(status_path):
        data = config.get_file(status_path)
    else:
        data = DEFAULT_DICT

    table_headers = ['File', 'Status']
    table_data = []

    # Check docker-compose.yml status
    text = _check_git(dash.path)
    table_data.append(['docker-compose.yml', text])

    # Files in config
    filedata = dash.config.get('files', [])
    for key in filedata:
        text = _check_git(filedata[key]['source_path'])
        if "ok" in text.lower():
            text = _check_git(filedata[key]['dest_path'])
            if "ok" in text.lower():
                text = _check_timestamp(filedata[key], dash.config['file_path'])
        table_data.append([filedata[key]['name'], text])

    # Save status.yml
    config.save_file(status_path, data)

    table_data.sort(key=lambda x: x[0])

    final_text = tabulate(
        table_data,
        table_headers
    )
    return dashing.Text(final_text, color=6, border_color=5,
                        title="Check Files")



import requests
from buzio import console, formatStr
from dashing import dashing
from tabulate import tabulate


def get_check_services(dash):
    """Check Docker-Compose and Ngrok status.

    Returns
    -------
        obj: dashing.Text

    """
    table_headers = ['Service', 'Status']
    table_data = []
    # Check Ngrok
    if dash.check_ngrok:
        try:
            ret = requests.get(
                "http://127.0.0.1:4040/api/tunnels", timeout=1)
            if ret.status_code == 200:
                text = formatStr.success("OK",
                                         use_prefix=False)
            else:
                text = formatStr.error("ERROR",
                                       use_prefix=False)
        except KeyboardInterrupt:
            raise KeyboardInterrupt
        except BaseException:
            text = formatStr.error("NOT RUNNING",
                                   use_prefix=False)
    table_data.append(['Ngrok', text])

    final_text = tabulate(
        table_data,
        table_headers
    )
    return dashing.Text(final_text, color=6, border_color=5, title="3rd Services Status")



"""Yaml Parser Module."""
import yaml
import sys
from buzio import console
import os
from cabrita.core.utils import get_path


class Config():
    """Configuration Class.

    Get and parse config from yaml configuration file
    defined in CABRITA_PATH environment settings or
    passed as parameter during initialization.
    """

    def get_file(self, path):
        """Convert configuration yaml file to python dict."""
        try:
            full_path = get_path(path, str(os.getcwd()))
            with open(full_path, 'r') as file:
                data = yaml.load(file.read())
                data['file_path'] = path
            return data
        except IOError as exc:
            console.error("Cannot open file: {}".format(exc))
            sys.exit(1)
        except yaml.YAMLError as exc:
            console.error("Cannot read file: {}".format(exc))
            sys.exit(1)
        except Exception as exc:
            console.error("Error: {}".format(exc))
            sys.exit(1)

    def save_file(self, path, content):
        try:
            full_path = get_path(path, str(os.getcwd()))
            with open(full_path, 'w') as file:
                yaml.dump(content, file)
        except IOError as exc:
            console.error("Cannot open file: {}".format(exc))
            sys.exit(1)
        except yaml.YAMLError as exc:
            console.error("Cannot read file: {}".format(exc))
            sys.exit(1)
        except Exception as exc:
            console.error("Error: {}".format(exc))
            sys.exit(1)

    def check_file(self, path):
        full_path = get_path(path, str(os.getcwd()))
        return os.path.isfile(full_path)



import psutil
from dashing import dashing


def get_info():
    """Get machine info using PSUtil.

    Returns
    -------
        obj: dashing.HSplit instance

    """
    cpu_percent = round(psutil.cpu_percent(interval=None) * 10, 0) / 10
    free_memory = int(psutil.virtual_memory().available / 1024 / 1024)
    total_memory = int(psutil.virtual_memory().total / 1024 / 1024)
    memory_percent = (free_memory / total_memory) * 100
    free_space = round(psutil.disk_usage("/").free / 1024 / 1024 / 1024, 1)
    total_space = round(psutil.disk_usage(
        "/").total / 1024 / 1024 / 1024, 1)
    space_percent = (free_space / total_space) * 100

    if memory_percent > 100:
        memory_percent = 100

    if space_percent > 100:
        space_percent = 100

    if cpu_percent <= 50:
        cpu_color = 2
    elif cpu_percent <= 70:
        cpu_color = 3
    else:
        cpu_color = 1

    if memory_percent <= 20:
        memory_color = 1
    elif memory_percent <= 50:
        memory_color = 3
    else:
        memory_color = 2

    if space_percent <= 20:
        space_color = 1
    elif space_percent <= 50:
        space_color = 3
    else:
        space_color = 2

    widget = dashing.HSplit(
        dashing.HGauge(
            val=cpu_percent,
            color=cpu_color,
            border_color=cpu_color,
            title="CPU:{}%".format(cpu_percent)
        ),
        dashing.HGauge(
            val=memory_percent,
            color=memory_color,
            border_color=memory_color,
            title="Free Mem:{}M".format(free_memory)
        ),
        dashing.HGauge(
            val=space_percent,
            color=space_color,
            border_color=space_color,
            title="Free Space:{}Gb".format(free_space)
        )
    )
    return widget
